@@ -199,7 +199,7 @@ process_unix_exec(process_t *process)
                "Cannot find maximum file descriptor, assuming: %d", max_fd);
     }
   }
-#else /* !(defined(_SC_OPEN_MAX)) */
+#else /* !defined(_SC_OPEN_MAX) */
   max_fd = DEFAULT_MAX_FD;
 #endif /* defined(_SC_OPEN_MAX) */
 
@@ -702,4 +702,4 @@ process_unix_close_file_descriptors(process_unix_t *unix_process)
   return success;
 }
 
-#endif /* defined(_WIN32). */
+#endif /* !defined(_WIN32) */
